/*
 * Copyright (c) 2005, 2013, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */
package org.openjdk.jmh;

import org.openjdk.jmh.profile.LinuxPerfAsmProfiler;
import org.openjdk.jmh.profile.ProfilerException;
import org.openjdk.jmh.results.Result;
import org.openjdk.jmh.runner.*;
import org.openjdk.jmh.util.Utils;

import java.io.File;
import java.io.IOException;

/**
 * Main program entry point
 */
public class PerfAsm {

    public static void main(String[] argv) throws RunnerException, IOException, ProfilerException {
        LinuxPerfAsmProfiler perf = new LinuxPerfAsmProfiler(argv.length > 0 ? argv[0] : "");

        if (argv.length < 1) {
            System.out.println(Utils.join(perf.addJVMInvokeOptions(null), " "));
            System.out.println(Utils.join(perf.addJVMOptions(null), " "));
        }

        for (Result r : perf.afterTrial(null, 0, new File(argv[1]), new File(argv[2]))) {
            System.out.println(r.extendedInfo());
        }
    }

}
